@@ -19,7 +19,7 @@
 # developers@gns3.net
 #
 
-# This file is for pylupdate4, to allow us to translate strings that we 
+# This file is for pylupdate4, to allow us to translate strings that we
 # use dynamically in GNS3 with Qtlinguist
 
 # Translation of the current language name
@@ -47,4 +47,5 @@ translate('nodesDock', 'EtherSwitch router')
 translate('nodesDock', 'Juniper router')
 translate('nodesDock', 'ASA firewall')
 translate('nodesDock', 'IDS')
-translate('nodesDock', 'Qemu host')
+translate('nodesDock', 'Qemu guest')
+translate('nodesDock', 'VirtualBox guest')
