@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 from GNS3.Node.AbstractNode import AbstractNode
@@ -40,13 +40,13 @@ class FRSW(AbstractNode):
 
         # assign a new hostname
         global frsw_id
-        
+
         # check if hostname has already been assigned
         for node in globals.GApp.topology.nodes.itervalues():
             if 'FR' + str(frsw_id) == node.hostname:
                 frsw_id = frsw_id + 1
                 break
-        
+
         self.hostname = 'FR' + str(frsw_id)
         frsw_id = frsw_id + 1
         AbstractNode.setCustomToolTip(self)
@@ -71,7 +71,8 @@ class FRSW(AbstractNode):
         if self.frsw:
             try:
                 self.frsw.delete()
-                del self.dynagen.devices[self.hostname]
+                if self.dynagen.devices.has_key(self.hostname):
+                    del self.dynagen.devices[self.hostname]
                 if self.frsw in self.hypervisor.devices:
                     self.hypervisor.devices.remove(self.frsw)
                 self.dynagen.update_running_config()
@@ -86,7 +87,7 @@ class FRSW(AbstractNode):
         self.hostname = hostname
         self.f = 'FRSW ' + self.hostname
         self.updateToolTips()
-        
+
     def setCustomToolTip(self):
         """ Set a custom tool tip
         """
@@ -119,12 +120,12 @@ class FRSW(AbstractNode):
     def duplicate_config(self):
         """ Returns a copy of the configuration
         """
-        
+
         config = self.config.copy()
         config['ports'] = list(self.config['ports'])
         config['mapping'] = self.config['mapping'].copy()
         return (config)
-        
+
     def set_config(self, config):
         """ Set a configuration in Dynamips
             config: dict
@@ -194,7 +195,7 @@ class FRSW(AbstractNode):
     def mapping(self):
         """ Configure Frame-relay DLCI mapping
         """
-        
+
         connected_interfaces = map(int, self.getConnectedInterfaceList())
         for (source,  destination) in self.config['mapping'].iteritems():
             (srcport, srcdlci) = source.split(':')
@@ -208,7 +209,7 @@ class FRSW(AbstractNode):
     def startNode(self):
         """ Start the node
         """
-        
+
         self.mapping()
         self.startupInterfaces()
         self.state = 'running'
