@@ -16,17 +16,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
-import os
-import sys
-import time
+#This class is used to start "qemuwrapper" automatically on localhost.
+#It is not used, if you start wrapper manually.
+
+import os, sys, time, socket
 import GNS3.Globals as globals
 import GNS3.Dynagen.qemu_lib as qlib
-from socket import socket, AF_INET, SOCK_STREAM
 from PyQt4 import QtCore, QtGui
 from GNS3.Utils import translate, debug, killAll
+from __main__ import VERSION
 
 class QemuManager(object):
     """ QemuManager class
@@ -45,7 +46,7 @@ class QemuManager(object):
         if self.proc:
             self.proc.kill()
 
-    def waitQemu(self):
+    def waitQemu(self, binding):
         """ Wait Qemu until it accepts connections
         """
 
@@ -53,13 +54,11 @@ class QemuManager(object):
         count = 15
         progress = None
         connection_success = False
-        debug("Qemu manager: connect on " + str(self.port))
+        timeout = 60.0
+        debug("Qemu manager: connecting to %s on port %i" % (binding, self.port))
         for nb in range(count + 1):
-            s = socket(AF_INET, SOCK_STREAM)
-            s.setblocking(0)
-            s.settimeout(300)
             if nb == 3:
-                progress = QtGui.QProgressDialog(unicode(translate("QemuManager", "Connecting to Qemu on port %i ...")) % self.port, 
+                progress = QtGui.QProgressDialog(translate("QemuManager", "Connecting to Qemu on %s port %i ...") % (binding, self.port),
                                                  translate("QemuManager", "Abort"), 0, count, globals.GApp.mainWindow)
                 progress.setMinimum(1)
                 progress.setWindowModality(QtCore.Qt.WindowModal)
@@ -71,9 +70,8 @@ class QemuManager(object):
                     progress.reset()
                     break
             try:
-                s.connect(('localhost', self.port))
+                s = socket.create_connection((binding, self.port), timeout)
             except:
-                s.close()
                 time.sleep(1)
                 continue
             connection_success = True
@@ -84,7 +82,7 @@ class QemuManager(object):
             time.sleep(0.2)
         else:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Qemu Manager',
-                                       unicode(translate("QemuManager", "Can't connect to Qemu on port %i")) % self.port)
+                                       translate("QemuManager", "Can't connect to Qemu on %s port %i") % (binding, self.port))
             self.stopQemu()
             return False
         if progress:
@@ -93,50 +91,53 @@ class QemuManager(object):
             progress = None
         return True
 
-    def startQemu(self, port):
+    def startQemu(self, port, binding=None):
         """ Start Qemu
         """
 
+        if binding == None:
+            binding = globals.GApp.systconf['qemu'].QemuManager_binding
         self.port = port
         if self.proc and self.proc.state():
-            debug('QemuManager: Qemu is already started with pid ' + str(self.proc.pid()))
+            debug("QemuManager: Qemu is already started with pid %i" % int(self.proc.pid()))
             return True
 
         self.proc = QtCore.QProcess(globals.GApp.mainWindow)
         if globals.GApp.systconf['qemu'].qemuwrapper_workdir:
             if not os.access(globals.GApp.systconf['qemu'].qemuwrapper_workdir, os.F_OK | os.W_OK):
-                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'Qemu Manager', 
-                                          unicode(translate("QemuManager", "Working directory %s seems to not exist or be writable, please check")) % globals.GApp.systconf['qemu'].qemuwrapper_workdir)
-            # set the working directory
+                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'Qemu Manager',
+                                          translate("QemuManager", "Working directory %s seems to not exist or be writable, please check") % globals.GApp.systconf['qemu'].qemuwrapper_workdir)
+
             self.proc.setWorkingDirectory(globals.GApp.systconf['qemu'].qemuwrapper_workdir)
 
         # test if Qemu is already running on this port
-        s = socket(AF_INET, SOCK_STREAM)
-        s.setblocking(0)
-        s.settimeout(300)
+        timeout = 60.0
         try:
-            s.connect(('localhost', self.port))
+            s = socket.create_connection((binding, self.port), timeout)
             QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'Qemu Manager',
-                                       unicode(translate("QemuManager", "Qemu is already running on port %i, it will not be shutdown after you quit GNS3")) % self.port)
+                                      translate("QemuManager", "Qemu is already running on %s port %i, it will not be shutdown after you quit GNS3") % (binding, self.port))
             s.close()
             return True
         except:
-            s.close()
+            pass
 
-        # start Qemuwrapper, use python on all platform but Windows
+        # start Qemuwrapper, use python on all platform but Windows (in release mode)
         binding = globals.GApp.systconf['qemu'].QemuManager_binding
-        if sys.platform.startswith('win'):
-            self.proc.start('"' + globals.GApp.systconf['qemu'].qemuwrapper_path + '"', ['--listen', binding, '--port', str(self.port)])
+        #print "ADEBUG: qemuwrapper_path = %s" % str(globals.GApp.systconf['qemu'].qemuwrapper_path)
+        if sys.platform.startswith('win') and (globals.GApp.systconf['qemu'].qemuwrapper_path.split('.')[-1] == 'exe'):
+            self.proc.start('"' + globals.GApp.systconf['qemu'].qemuwrapper_path + '"', ['--listen', binding, '--port', str(self.port), '--no-path-check'])
+        elif hasattr(sys, "frozen"):
+            self.proc.start('python',  [globals.GApp.systconf['qemu'].qemuwrapper_path, '--listen', binding, '--port', str(self.port), '--no-path-check'])
         else:
-            self.proc.start('python',  [globals.GApp.systconf['qemu'].qemuwrapper_path, '--listen', binding, '--port', str(self.port)])
+            self.proc.start(sys.executable,  [globals.GApp.systconf['qemu'].qemuwrapper_path, '--listen', binding, '--port', str(self.port), '--no-path-check'])
 
         if self.proc.waitForStarted() == False:
-            QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Qemu Manager',  unicode(translate("QemuManager", "Can't start Qemu on port %i")) % self.port)
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'Qemu Manager', translate("QemuManager", "Can't start Qemu on %s port %i") % (binding, self.port))
             return False
 
-        self.waitQemu()
+        self.waitQemu(binding)
         if self.proc and self.proc.state():
-            debug('QemuManager: Qemu has been started with pid ' + str(self.proc.pid()))
+            debug("QemuManager: Qemu has been started with pid %i"  % int(self.proc.pid()))
         return True
 
     def stopQemu(self):
@@ -151,30 +152,35 @@ class QemuManager(object):
                 except:
                     continue
         if self.proc and self.proc.state():
-            debug('QemuManager: stop Qemu with pid ' + str(self.proc.pid()))
+            debug("QemuManager: stop Qemu with pid %i" % int(self.proc.pid()))
+            self.proc.terminate()
+            time.sleep(0.5)
             self.proc.close()
         self.proc = None
-        
+
     def preloadQemuwrapper(self):
         """ Preload Qemuwrapper
         """
 
         proc = QtCore.QProcess(globals.GApp.mainWindow)
+        binding = globals.GApp.systconf['qemu'].QemuManager_binding
 
         if globals.GApp.systconf['qemu'].qemuwrapper_workdir:
             if not os.access(globals.GApp.systconf['qemu'].qemuwrapper_workdir, os.F_OK | os.W_OK):
-                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'Qemu Manager', 
-                                          unicode(translate("QemuManager", "Working directory %s seems to not exist or be writable, please check")) % globals.GApp.systconf['qemu'].qemuwrapper_workdir)
+                QtGui.QMessageBox.warning(globals.GApp.mainWindow, 'Qemu Manager',
+                                          translate("QemuManager", "Working directory %s seems to not exist or be writable, please check") % globals.GApp.systconf['qemu'].qemuwrapper_workdir)
                 return False
-            # set the working directory
+
             proc.setWorkingDirectory(globals.GApp.systconf['qemu'].qemuwrapper_workdir)
-        
-        
-        # start Qemuwrapper, use python on all platform but Windows
-        if sys.platform.startswith('win'):
-            proc.start('"' + globals.GApp.systconf['qemu'].qemuwrapper_path + '"')
+
+        # start Qemuwrapper, use python on all platform but Windows (in release mode)
+        if sys.platform.startswith('win') and (globals.GApp.systconf['qemu'].qemuwrapper_path.split('.')[-1] == 'exe'):
+            # On Windows hosts, we remove python dependency by pre-compiling Qemuwrapper. (release mode)
+            proc.start('"' + globals.GApp.systconf['qemu'].qemuwrapper_path + '"', ['--listen', binding, '--no-path-check'])
+        elif hasattr(sys, "frozen"):
+            proc.start('python',  [globals.GApp.systconf['qemu'].qemuwrapper_path, '--listen', binding, '--no-path-check'])
         else:
-            proc.start('python',  [globals.GApp.systconf['qemu'].qemuwrapper_path])
+            proc.start(sys.executable,  [globals.GApp.systconf['qemu'].qemuwrapper_path, '--listen', binding, '--no-path-check'])
 
         if proc.waitForStarted() == False:
             return False
@@ -182,20 +188,32 @@ class QemuManager(object):
         # give 5 seconds to the hypervisor to accept connections
         count = 5
         connection_success = False
+        timeout = 60.0
         for nb in range(count + 1):
-            s = socket(AF_INET, SOCK_STREAM)
-            s.setblocking(0)
-            s.settimeout(300)
             try:
-                s.connect(('localhost', self.port))
+                s = socket.create_connection((binding, self.port), timeout)
             except:
-                s.close()
                 time.sleep(1)
                 continue
             connection_success = True
             break
+        s.close()
         if connection_success:
-            s.close()
+            # check qemuwrapper version
+            proc.waitForReadyRead(5000)
+            output = proc.readAllStandardOutput()
+            ver = QtCore.QByteArray('(version ')
+            verOffset = output.indexOf(ver) + len('(version ')
+            if verOffset != -1:
+                ver = QtCore.QByteArray(")" + os.linesep)
+                endVerOffset = output.indexOf(ver, verOffset) - verOffset
+                wrapperVer = output.mid(verOffset, endVerOffset)
+                if wrapperVer != VERSION:
+                    print 'QemuManager: qemuwrapper version check failed: (' + wrapperVer  + '|' + VERSION + ')'
+                    print 'QemuManager: please update your qemuwrapper and check its path in the settings'
+                    proc.close()
+                    return False
+
             proc.close()
             return True
         if proc.state():
