@@ -16,9 +16,21 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
+#This module is responsible for all the buttons (and reactions) on the main topology.
+
+#debuglevel: 0=disabled, 1=default, 2=debug, 3=deep debug
+debuglevel = 0
+
+def debugmsg(level, message):
+    if debuglevel == 0:
+        return
+    if debuglevel >= level:
+        print message
+
+import os, sys, time
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.UndoFramework as undo
@@ -36,7 +48,8 @@ from GNS3.NodeConfigurator import NodeConfigurator
 from GNS3.Node.AbstractNode import AbstractNode
 from GNS3.Globals.Symbols import SYMBOLS, SYMBOL_TYPES
 from GNS3.Node.IOSRouter import IOSRouter
-from GNS3.Node.AnyEmuDevice import AnyEmuDevice
+from GNS3.Node.AnyEmuDevice import AnyEmuDevice, PIX
+from GNS3.Node.AnyVBoxEmuDevice import AnyVBoxEmuDevice
 from GNS3.Node.FRSW import FRSW
 from GNS3.Node.ATMSW import ATMSW
 from GNS3.Node.ETHSW import ETHSW
@@ -67,14 +80,16 @@ class Scene(QtGui.QGraphicsView):
         self.newedge = None
         self.resetAddingLink()
         self.reloadRenderers()
-        
+
         self.sceneDragging = False
-        self.lastMousePos = None 
+        self.lastMousePos = None
+        
+        self.showWarningMessageBackgroundLayer = True
 
     def reloadRenderers(self):
         """ Load all needed renderers
         """
-            
+
         # Load all renders
         self.renders = {}
         for item in SYMBOLS:
@@ -96,16 +111,12 @@ class Scene(QtGui.QGraphicsView):
             self.__topology.removeItem(self.newedge)
             self.newedge = None
 
-    def showContextualMenu(self):
-        """  Create and display a contextual menu when clicking on the view
-        """
+    def makeContextualMenu(self, menu):
 
         items = self.__topology.selectedItems()
         if len(items) == 0:
             return
 
-        menu = QtGui.QMenu()
-
         instances = map(lambda item: not isinstance(item, Annotation) and not isinstance(item, Pixmap) and not isinstance(item, AbstractShapeItem), items)
         if True in instances:
 
@@ -130,32 +141,32 @@ class Scene(QtGui.QGraphicsView):
 
         instances = map(lambda item: isinstance(item, ETHSW) or isinstance(item, ATMSW) or isinstance(item, ATMBR) or isinstance(item, FRSW), items)
         if True in instances:
-            
+
             # Action: ChangeHypervisor (Change the hypervisor)
             changeHypervisor = QtGui.QAction(translate('Scene', 'Set an hypervisor'), menu)
             changeHypervisor.setIcon(QtGui.QIcon(":/icons/applications.svg"))
             self.connect(changeHypervisor, QtCore.SIGNAL('triggered()'), self.slotChangeHypervisor)
-            
+
             menu.addAction(changeHypervisor)
-            
+
         instances = map(lambda item: isinstance(item, ETHSW), items)
         if True in instances:
-            
+
             # Action: MAC Table
             MACTableAct = QtGui.QAction(translate('Scene', 'MAC Address Table'), menu)
             MACTableAct.setIcon(QtGui.QIcon(':/icons/inspect.svg'))
             self.connect(MACTableAct, QtCore.SIGNAL('triggered()'), self.slotMACTable)
 
             menu.addAction(MACTableAct)
-            
+
         instances = map(lambda item: isinstance(item, AbstractNode), items)
         if True in instances:
 
             # Action: Change symbol
             changeSymbol = QtGui.QAction(translate('Scene', 'Change Symbol'), menu)
             changeSymbol.setIcon(QtGui.QIcon(':/icons/node_conception.svg'))
-            self.connect(changeSymbol, QtCore.SIGNAL('triggered()'), self.slotchangeSymbol)        
-            menu.addAction(changeSymbol)            
+            self.connect(changeSymbol, QtCore.SIGNAL('triggered()'), self.slotchangeSymbol)
+            menu.addAction(changeSymbol)
 
         instances = map(lambda item: isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice), items)
         if True in instances:
@@ -164,30 +175,89 @@ class Scene(QtGui.QGraphicsView):
             consolePortAct = QtGui.QAction(translate('Scene', 'Change console port'), menu)
             consolePortAct.setIcon(QtGui.QIcon(':/icons/console_port.svg'))
             self.connect(consolePortAct, QtCore.SIGNAL('triggered()'), self.slotChangeConsolePort)
-            
+
             # Action: Console (Connect to the node console)
             consoleAct = QtGui.QAction(translate('Scene', 'Console'), menu)
             consoleAct.setIcon(QtGui.QIcon(':/icons/console.svg'))
             self.connect(consoleAct, QtCore.SIGNAL('triggered()'), self.slotConsole)
 
+            # Action: Capture traffic on an interface
+            captureAct = QtGui.QAction(translate('Scene', 'Capture'), menu)
+            captureAct.setIcon(QtGui.QIcon(':/icons/inspect.svg'))
+            self.connect(captureAct, QtCore.SIGNAL('triggered()'), self.slotCapture)
+
+            menu.addAction(consolePortAct)
+            menu.addAction(consoleAct)
+            menu.addAction(captureAct)
+
+        instances = map(lambda item: isinstance(item, AnyVBoxEmuDevice), items)
+        if True in instances:
+
+            # Action: Bring window to front
+            displayWindowFocusAct = QtGui.QAction(translate('Scene', 'Bring display to front'), menu)
+            displayWindowFocusAct.setIcon(QtGui.QIcon(':/symbols/computer.normal.svg'))
+            self.connect(displayWindowFocusAct, QtCore.SIGNAL('triggered()'), self.slotDisplayWindowFocus)
+            menu.addAction(displayWindowFocusAct)
+
+            if not sys.platform.startswith('darwin'):
+                # Only if not OSX
+                # Action: Hide window
+                displayWindowHideAct = QtGui.QAction(translate('Scene', 'Hide display window'), menu)
+                displayWindowHideAct.setIcon(QtGui.QIcon(':/symbols/computer.normal.svg'))
+                self.connect(displayWindowHideAct, QtCore.SIGNAL('triggered()'), self.slotDisplayWindowHide)
+                menu.addAction(displayWindowHideAct)
+                
+            # Action: Change the console port
+            consolePortAct = QtGui.QAction(translate('Scene', 'Change console port'), menu)
+            consolePortAct.setIcon(QtGui.QIcon(':/icons/console_port.svg'))
+            self.connect(consolePortAct, QtCore.SIGNAL('triggered()'), self.slotChangeConsolePort)
+            menu.addAction(consolePortAct)
+
+            # Action: Console (Connect to the node console)
+            consoleAct = QtGui.QAction(translate('Scene', 'Console'), menu)
+            consoleAct.setIcon(QtGui.QIcon(':/icons/console.svg'))
+            self.connect(consoleAct, QtCore.SIGNAL('triggered()'), self.slotConsole)
+            menu.addAction(consoleAct)
+
+            # Action: Capture traffic on an interface
+            captureAct = QtGui.QAction(translate('Scene', 'Capture'), menu)
+            captureAct.setIcon(QtGui.QIcon(':/icons/inspect.svg'))
+            self.connect(captureAct, QtCore.SIGNAL('triggered()'), self.slotCapture)
+            menu.addAction(captureAct)
+
+        instances = map(lambda item: isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice), items)
+        if True in instances:
+
             # Action: Start (Start IOS on hypervisor)
             startAct = QtGui.QAction(translate('Scene', 'Start'), menu)
             startAct.setIcon(QtGui.QIcon(':/icons/play.svg'))
             self.connect(startAct, QtCore.SIGNAL('triggered()'), self.slotStartNode)
 
+            menu.addAction(startAct)
+
+        instances = map(lambda item: isinstance(item, IOSRouter) or isinstance(item, AnyVBoxEmuDevice), items)
+        if True in instances:
+
+            # Action: Suspend / Pause (Suspend IOS on hypervisor/pause VM)
+            suspendAct = QtGui.QAction(translate('Scene', 'Suspend'), menu)
+            suspendAct.setIcon(QtGui.QIcon(':/icons/pause.svg'))
+            self.connect(suspendAct, QtCore.SIGNAL('triggered()'), self.slotSuspendNode)
+
+            menu.addAction(suspendAct)
+
+        instances = map(lambda item: isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice), items)
+        if True in instances:
+
             # Action: Stop (Stop IOS on hypervisor)
             stopAct = QtGui.QAction(translate('Scene', 'Stop'), menu)
             stopAct.setIcon(QtGui.QIcon(':/icons/stop.svg'))
             self.connect(stopAct, QtCore.SIGNAL('triggered()'), self.slotStopNode)
-            
+
             # Action: Reload (stop and start IOS on hypervisor)
             reloadAct = QtGui.QAction(translate('Scene', 'Reload'), menu)
             reloadAct.setIcon(QtGui.QIcon(':/icons/reload.svg'))
             self.connect(reloadAct, QtCore.SIGNAL('triggered()'), self.slotReloadNode)
 
-            menu.addAction(consolePortAct)
-            menu.addAction(consoleAct)
-            menu.addAction(startAct)
             menu.addAction(stopAct)
             menu.addAction(reloadAct)
 
@@ -201,25 +271,19 @@ class Scene(QtGui.QGraphicsView):
 
             # Action: Console (Connect to the node console)
             AuxAct = QtGui.QAction(translate('Scene', 'Console to AUX port'), menu)
-            AuxAct.setIcon(QtGui.QIcon(':/icons/aux.svg'))
+            AuxAct.setIcon(QtGui.QIcon(':/icons/aux-console.svg'))
             self.connect(AuxAct, QtCore.SIGNAL('triggered()'), self.slotAuxConsole)
 
             # Action: Calculate IDLE PC
             idlepcAct = QtGui.QAction(translate('Scene', 'Idle PC'), menu)
             idlepcAct.setIcon(QtGui.QIcon(':/icons/calculate.svg'))
             self.connect(idlepcAct, QtCore.SIGNAL('triggered()'), self.slotIdlepc)
-            
+
             # Action: Change the startup-config
             StartupConfigAct = QtGui.QAction(translate('Scene', 'Startup-config'), menu)
             StartupConfigAct.setIcon(QtGui.QIcon(':/icons/startup_config.svg'))
             self.connect(StartupConfigAct, QtCore.SIGNAL('triggered()'), self.slotStartupConfig)
 
-            # Action: Suspend (Suspend IOS on hypervisor)
-            suspendAct = QtGui.QAction(translate('Scene', 'Suspend'), menu)
-            suspendAct.setIcon(QtGui.QIcon(':/icons/pause.svg'))
-            self.connect(suspendAct, QtCore.SIGNAL('triggered()'), self.slotSuspendNode)
-
-            menu.addAction(suspendAct)
             menu.addAction(auxPortAct)
             menu.addAction(AuxAct)
             menu.addAction(idlepcAct)
@@ -227,60 +291,67 @@ class Scene(QtGui.QGraphicsView):
 
         instances = map(lambda item: isinstance(item, Annotation) or isinstance(item, AbstractShapeItem), items)
         if True in instances:
-        
+
             # Action: Style
             styleAct = QtGui.QAction(translate('Scene', 'Style'), menu)
             styleAct.setIcon(QtGui.QIcon(':/icons/drawing.svg'))
             self.connect(styleAct, QtCore.SIGNAL('triggered()'), self.slotStyle)
 
             menu.addAction(styleAct)
-            
+
             # Action: Duplicate
             duplicateAct = QtGui.QAction(translate('Scene', 'Duplicate'), menu)
             duplicateAct.setIcon(QtGui.QIcon(':/icons/new.svg'))
             self.connect(duplicateAct, QtCore.SIGNAL('triggered()'), self.slotDuplicate)
 
             menu.addAction(duplicateAct)
-            
+
         # Action: Delete (Delete the node)
         deleteAct = QtGui.QAction(translate('Scene', 'Delete'), menu)
         deleteAct.setIcon(QtGui.QIcon(':/icons/delete.svg'))
         self.connect(deleteAct, QtCore.SIGNAL('triggered()'), self.slotDeleteNode)
         menu.addAction(deleteAct)
-        
+
         # Action: Lower Z value
-        lowerZvalueAct = QtGui.QAction(translate('Scene', 'Lower one step'), menu)
+        lowerZvalueAct = QtGui.QAction(translate('Scene', 'Lower one layer'), menu)
         lowerZvalueAct.setIcon(QtGui.QIcon(':/icons/lower_z_value.svg'))
         self.connect(lowerZvalueAct, QtCore.SIGNAL('triggered()'), self.slotlowerZValue)
-            
+
         # Action: Raise Z value
-        raiseZvalueAct = QtGui.QAction(translate('Scene', 'Raise one step'), menu)
+        raiseZvalueAct = QtGui.QAction(translate('Scene', 'Raise one layer'), menu)
         raiseZvalueAct.setIcon(QtGui.QIcon(':/icons/raise_z_value.svg'))
         self.connect(raiseZvalueAct, QtCore.SIGNAL('triggered()'), self.slotraiseZValue)
-        
+
         menu.addAction(raiseZvalueAct)
         menu.addAction(lowerZvalueAct)
 
         items = self.__topology.selectedItems()
         if len(items) > 1:
-       
+
             # Action: Align horizontally
             hozAlignAct = QtGui.QAction(translate('Scene', 'Align horizontally'), menu)
             hozAlignAct.setIcon(QtGui.QIcon(':/icons/horizontally.svg'))
             self.connect(hozAlignAct, QtCore.SIGNAL('triggered()'), self.slotHozAlignment)
             menu.addAction(hozAlignAct)
-            
+
             # Action: Align vertically
             vertAlignAct = QtGui.QAction(translate('Scene', 'Align vertically'), menu)
             vertAlignAct.setIcon(QtGui.QIcon(':/icons/vertically.svg'))
             self.connect(vertAlignAct, QtCore.SIGNAL('triggered()'), self.slotVertAlignment)
             menu.addAction(vertAlignAct)
 
+    def showContextualMenu(self):
+        """  Create and display a contextual menu when clicking on the view
+        """
+
+        menu = QtGui.QMenu()
+        self.makeContextualMenu(menu)
         menu.exec_(QtGui.QCursor.pos())
+        menu.clear()
 
         # force the deletion of the children
-        for child in menu.children():
-            child.deleteLater()
+        # for child in menu.children():
+        #    child.deleteLater()
 
     def addItem(self, node):
         """ Overloaded function that add the node into the topology
@@ -294,7 +365,7 @@ class Scene(QtGui.QGraphicsView):
 
         splash = QtGui.QSplashScreen(QtGui.QPixmap(":images/logo_gns3_splash.png"))
         splash.show()
-        splash.showMessage(translate("Scene", "Please wait while calculating IDLE PC values"))
+        splash.showMessage(translate("Scene", "Please wait while calculating an IDLE PC"))
         globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
         result = globals.GApp.dynagen.devices[router.hostname].idleprop(lib.IDLEPROPGET)
         return result
@@ -305,22 +376,22 @@ class Scene(QtGui.QGraphicsView):
 
         for item in self.__topology.selectedItems():
             if isinstance(item, ETHSW):
-                table = MACTableDialog(item)
+                table = MACTableDialog(item, globals.GApp.mainWindow)
                 table.show()
                 table.exec_()
 
     def slotHozAlignment(self):
         """ Horizontally align items
         """
-        
+
         hozPos = self.__topology.selectedItems()[0].y()
         for item in self.__topology.selectedItems():
             item.setPos(item.x(), hozPos)
-            
+
     def slotVertAlignment(self):
         """ Vertically align items
         """
-        
+
         vertPos = self.__topology.selectedItems()[0].x()
         for item in self.__topology.selectedItems():
             item.setPos(vertPos, item.y())
@@ -328,7 +399,7 @@ class Scene(QtGui.QGraphicsView):
     def slotStyle(self):
         """ Change the style of an annotation or a shape item
         """
-        
+
         style = StyleDialog()
         style.setModal(True)
         if self.__topology.selectedItems():
@@ -359,7 +430,7 @@ class Scene(QtGui.QGraphicsView):
     def slotDuplicate(self):
         """ Duplicate an item
         """
-        
+
         for item in self.__topology.selectedItems():
             if isinstance(item, Annotation):
                 dupnote = Annotation()
@@ -404,7 +475,7 @@ class Scene(QtGui.QGraphicsView):
         try:
             if globals.GApp.dynagen.devices[router.hostname].idlepc != None:
                 reply = QtGui.QMessageBox.question(globals.GApp.mainWindow,translate("Scene", "IDLE PC"),
-                                                   unicode(translate("Scene", "%s already has an idlepc value applied, do you want to calculate a new one?")) % router.hostname,
+                                                   translate("Scene", "%s already has an idlepc value applied, do you want to calculate a new one?") % router.hostname,
                                                    QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
                 if reply == QtGui.QMessageBox.Yes:
                     # reset idlepc
@@ -417,6 +488,9 @@ class Scene(QtGui.QGraphicsView):
         except lib.DynamipsError, msg:
             QtGui.QMessageBox.critical(self, translate("Scene", "Dynamips error"),  unicode(msg))
             return
+        except lib.DynamipsErrorHandled:
+            QtGui.QMessageBox.critical(self, translate("Scene", "Dynamips error"), translate("Scene", "Connection lost"))
+            return
 
         # remove the '100-OK' line
         result.pop()
@@ -438,13 +512,13 @@ class Scene(QtGui.QGraphicsView):
             options.append(option)
             idles[i] = value
             i += 1
-
         if len(idles) == 0:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "IDLE PC"),  translate("Scene", "No idlepc values found"))
             return
 
         idlepcdiag = IDLEPCDialog(router, idles, options)
         idlepcdiag.show()
+        idlepcdiag.raise_()
         idlepcdiag.exec_()
 
     def slotConfigNode(self):
@@ -466,7 +540,7 @@ class Scene(QtGui.QGraphicsView):
 
         for item in self.__topology.selectedItems():
             item.changeHostname()
-            
+
     def slotChangeHypervisor(self):
         """ Slot called to change hypervisor of selected items
         """
@@ -494,9 +568,9 @@ class Scene(QtGui.QGraphicsView):
         for item in self.__topology.selectedItems():
             if not isinstance(item, Annotation) and not isinstance(item, Pixmap) and not isinstance(item, AbstractShapeItem):
                 count += 1
-        
+
         if count > 1:
-            reply = QtGui.QMessageBox.question(self, translate("Scene", "Message"), translate("Scene", "Do you really want to delete these nodes?"), 
+            reply = QtGui.QMessageBox.question(self, translate("Scene", "Message"), translate("Scene", "Do you really want to delete these nodes?"),
                                             QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
 
             if reply == QtGui.QMessageBox.No:
@@ -505,6 +579,9 @@ class Scene(QtGui.QGraphicsView):
         ok_to_delete_node = True
         for item in self.__topology.selectedItems():
             if not isinstance(item, Annotation) and not isinstance(item, Pixmap) and not isinstance(item, AbstractShapeItem):
+                # forced stop for Qemu based devices (Qemuwrapper doesn't support hot link removal)
+                #if isinstance(item, AnyEmuDevice):
+                #    item.stopNode()
                 for link in item.getEdgeList().copy():
                     if self.__topology.deleteLinkFromScene(link) == False:
                         if ok_to_delete_node:
@@ -519,30 +596,33 @@ class Scene(QtGui.QGraphicsView):
     def slotlowerZValue(self):
         """ Lower Z value
         """
- 
+
         for item in self.__topology.selectedItems():
             zvalue = item.zValue()
             if zvalue > 0:
                 command = undo.NewZValue(item, zvalue - 1)
                 self.__topology.undoStack.push(command)
-            elif isinstance(item, AbstractShapeItem):
-                # shape items can have a z value lower than 0
+            elif isinstance(item, AbstractShapeItem) or isinstance(item, Annotation) or isinstance(item, Pixmap):
+                # shape items, annotations and pictures can have a z value lower than 0
                 command = undo.NewZValue(item, zvalue - 1)
                 self.__topology.undoStack.push(command)
+                if zvalue == 0 and self.showWarningMessageBackgroundLayer:
+                    QtGui.QMessageBox.information(globals.GApp.mainWindow, translate("Scene", "Layer position"),  translate("Scene", "Object moved to a background layer. You will now have to use the right-click action to select this object in the future and raise it to layer 0 to be able to move it"))
+                    self.showWarningMessageBackgroundLayer = False
 
     def slotraiseZValue(self):
         """ Raise Z value
         """
-    
+
         for item in self.__topology.selectedItems():
             zvalue = item.zValue()
             command = undo.NewZValue(item, zvalue + 1)
             self.__topology.undoStack.push(command)
-            
+
     def slotchangeSymbol(self):
         """ Change a device's symbol
         """
-        
+
         item_list = []
         for item in self.__topology.selectedItems():
             if isinstance(item, AbstractNode):
@@ -557,25 +637,61 @@ class Scene(QtGui.QGraphicsView):
         """
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice):
+                time.sleep(globals.GApp.systconf['general'].console_delay)
                 item.console()
-                
+
+    def slotCapture(self):
+        """ Slot called to capture on the selected items
+        """
+
+        for item in self.__topology.selectedItems():
+            if isinstance(item, IOSRouter) or (isinstance(item, AnyEmuDevice) and not isinstance(item, PIX)) or isinstance(item, AnyVBoxEmuDevice):
+                links = []
+                for localif in item.getConnectedInterfaceList():
+                    linkobj = item.getConnectedLinkByName(localif)
+                    (neighbor, neighborif) = linkobj.getConnectedNeighbor(item)
+                    links.append("%s connected to %s %s" % (localif, neighbor.hostname, neighborif))
+                if len(links):
+                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Scene", "Capture"), translate("Scene", "Please choose a link"), links, 0, False)
+                    if ok:
+                        interface = unicode(selection).split(' ')[0]
+                        linkobj = item.getConnectedLinkByName(interface)
+                        linkobj.startCapture()
+
+    def slotDisplayWindowFocus(self):
+        """ Slot called to bring VM's display as foreground window and focus on it
+        """
+        for item in self.__topology.selectedItems():
+            if isinstance(item, AnyVBoxEmuDevice):
+                item.displayWindowFocus()
+                return
+
+    def slotDisplayWindowHide(self):
+        """ Slot called to hide VM's display window
+        """
+        for item in self.__topology.selectedItems():
+            if isinstance(item, AnyVBoxEmuDevice):
+                item.displayWindowHide()
+                return
+
     def slotAuxConsole(self):
         """ Slot called to launch a console to AUX on the selected items
         """
 
         for item in self.__topology.selectedItems():
             if isinstance(item, IOSRouter):
+                time.sleep(globals.GApp.systconf['general'].console_delay)
                 item.aux()
-                
+
     def slotChangeConsolePort(self):
         """ Slot called to change the console port
         """
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice):
                 item.changeConsolePort()
-                
+
     def slotChangeAUXPort(self):
         """ Slot called to change the aux port
         """
@@ -597,27 +713,27 @@ class Scene(QtGui.QGraphicsView):
         """
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice):
                 item.startNode()
 
     def slotStopNode(self):
         """ Slot called to stop the selected items
         """
-     
+
         count = 0
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice):
                 count += 1
 
         if count > 1:
-            reply = QtGui.QMessageBox.question(self, translate("Scene", "Message"), translate("Scene", "Do you really want to stop these devices?"), 
+            reply = QtGui.QMessageBox.question(globals.GApp.mainWindow, translate("Scene", "Message"), translate("Scene", "Do you really want to stop these devices?"),
                                             QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
 
             if reply == QtGui.QMessageBox.No:
                 return
-     
+
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice):
                 item.stopNode()
 
     def slotSuspendNode(self):
@@ -625,35 +741,35 @@ class Scene(QtGui.QGraphicsView):
         """
 
         for item in self.__topology.selectedItems():
-            if  isinstance(item, IOSRouter):
+            if  isinstance(item, IOSRouter) or isinstance(item, AnyVBoxEmuDevice):
                 item.suspendNode()
-                
+
     def slotReloadNode(self):
         """ Slot called to reload the selected items
         """
 
         count = 0
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice):
                 count += 1
 
         if count > 1:
-            reply = QtGui.QMessageBox.question(self, translate("Scene", "Message"), translate("Scene", "Do you really want to reload these devices?"), 
+            reply = QtGui.QMessageBox.question(globals.GApp.mainWindow, translate("Scene", "Message"), translate("Scene", "Do you really want to reload these devices?"),
                                             QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
 
             if reply == QtGui.QMessageBox.No:
                 return
 
         for item in self.__topology.selectedItems():
-            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice):
+            if isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice) or isinstance(item, AnyVBoxEmuDevice):
                 item.reloadNode()
-                
+
     def getSourceNode(self):
-    
+
         return globals.GApp.topology.getNode(self.__sourceNodeID)
-        
+
     def getDestNode(self):
-    
+
         return globals.GApp.topology.getNode(self.__destNodeID)
 
     def __addLink(self):
@@ -674,7 +790,7 @@ class Scene(QtGui.QGraphicsView):
         # add the link into the topology
         if self.__topology.addLinkFromScene(self.__sourceNodeID, self.__sourceInterface, self.__destNodeID, self.__destInterface) == False:
             self.__isFirstClick = True
-        
+
         self.__sourceNodeID = None
         self.__destNodeID = None
 
@@ -683,11 +799,45 @@ class Scene(QtGui.QGraphicsView):
             id: integer
             interface: string
         """
+        debugmsg(2, "Scene.py: id = %s" % str(id))
+        debugmsg(2, "Scene.py: interface = %s" % str(interface))
+        debugmsg(2, "Scene.py: node = %s" % str(self.__topology.getNode(id)))
 
         if id == None and interface == None:
             self.__isFirstClick = True
             return
 
+        node = self.__topology.getNode(id)
+
+        if globals.currentLinkType == globals.Enum.LinkType.Serial or globals.currentLinkType == globals.Enum.LinkType.ATM or globals.currentLinkType == globals.Enum.LinkType.POS:
+            if isinstance(node, AnyEmuDevice) or isinstance(node, AnyVBoxEmuDevice) or isinstance(node, ETHSW):
+                if isinstance(node, AnyEmuDevice):
+                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "AddLink"),  translate("Scene", "Qemu machines support only Ethernet links."))
+                if isinstance(node, AnyVBoxEmuDevice):
+                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "AddLink"),  translate("Scene", "VirtualBox machines support only Ethernet links."))
+                if isinstance(node, ETHSW):
+                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "AddLink"),  translate("Scene", "Ethernet switch supports only Ethernet links."))
+                self.__isFirstClick = True
+                return
+
+        if not (globals.currentLinkType == globals.Enum.LinkType.ATM or \
+                globals.currentLinkType == globals.Enum.LinkType.Manual) and isinstance(node, ATMSW):
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "AddLink"),  translate("Scene", "ATM switch supports only ATM links."))
+            self.__isFirstClick = True
+            return
+
+        if not (globals.currentLinkType == globals.Enum.LinkType.Serial or \
+                globals.currentLinkType == globals.Enum.LinkType.Manual) and isinstance(node, FRSW):
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "AddLink"),  translate("Scene", "Frame-Relay switch supports only serial links."))
+            self.__isFirstClick = True
+            return
+
+        if (globals.currentLinkType == globals.Enum.LinkType.Serial or \
+            globals.currentLinkType == globals.Enum.LinkType.POS) and isinstance(node, ATMBR):
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "AddLink"),  translate("Scene", "ATM bridge supports only ATM and Ethernet links."))
+            self.__isFirstClick = True
+            return
+
         if globals.addingLinkFlag:
             # user is adding a link
             if self.__isFirstClick:
@@ -695,7 +845,7 @@ class Scene(QtGui.QGraphicsView):
                 self.__sourceNodeID = id
                 self.__sourceInterface = interface
                 self.__isFirstClick = False
-                node = self.__topology.getNode(id)
+                #node = self.__topology.getNode(id)
                 if (globals.currentLinkType == globals.Enum.LinkType.Serial or globals.currentLinkType == globals.Enum.LinkType.ATM) or \
                     (globals.currentLinkType == globals.Enum.LinkType.Manual and ((interface[0] == 's' or interface[0] == 'a') or (isinstance(node, ATMSW) or isinstance(node, FRSW)))):
                     # interface is serial or ATM
@@ -756,6 +906,9 @@ class Scene(QtGui.QGraphicsView):
                     return
             self.slotDeleteNode()
         elif globals.addingLinkFlag and key == QtCore.Qt.Key_Escape:
+            if self.__isFirstClick:
+                # Escape has been pressed while we were not drawing a link
+                globals.GApp.workspace.stopAction_addLink()
             self.resetAddingLink()
         else:
             QtGui.QGraphicsView.keyPressEvent(self, event)
@@ -764,14 +917,27 @@ class Scene(QtGui.QGraphicsView):
         """ Drag move event
         """
 
-        event.accept()
+        #debug("Drop event %s" % str(list(event.mimeData().formats())))
+        if event.mimeData().hasFormat("text/uri-list") or event.mimeData().hasFormat("application/x-qt-mime-type-name") :
+            event.acceptProposedAction()
 
     def dropEvent(self, event):
         """ Drop event
         """
 
-        if event.mimeData().hasText():
-
+        if event.mimeData().hasFormat("text/uri-list") and event.mimeData().hasUrls():
+            if len(event.mimeData().urls()) > 1:
+                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Scene", "Topology file"),  translate("Scene", "Please select only one file!"))
+                return
+            for url in event.mimeData().urls():
+                path = unicode(url.toLocalFile(), 'utf-8', errors='replace')
+                if os.path.isfile(path):
+                    path = os.path.normpath(path)
+                    debug("Load file from drop event %s" % path)
+                    globals.GApp.workspace.openFromDroppedFile(path)
+                    break
+            event.acceptProposedAction()
+        elif event.mimeData().hasText():
             symbolname = str(event.mimeData().text())
             # Get resource corresponding to node type
             object = None
@@ -800,8 +966,9 @@ class Scene(QtGui.QGraphicsView):
             pos_y = node.pos().y() - (node.boundingRect().height() / 2)
             node.setPos(pos_x, pos_y)
 
-            event.setDropAction(QtCore.Qt.MoveAction)
-            event.accept()
+            event.acceptProposedAction()
+            #event.setDropAction(QtCore.Qt.MoveAction)
+            #event.accept()
         else:
             event.ignore()
 
@@ -824,8 +991,8 @@ class Scene(QtGui.QGraphicsView):
             self.lastMousePos = self.mapFromGlobal(event.globalPos())
             self.sceneDragging = True
             globals.GApp.scene.setCursor(QtCore.Qt.ClosedHandCursor)
-            return 
-                
+            return
+
         if show and event.modifiers() & QtCore.Qt.ShiftModifier and event.button() == QtCore.Qt.LeftButton and item and not globals.addingLinkFlag:
             item.setSelected(True)
         elif show and event.button() == QtCore.Qt.RightButton and not globals.addingLinkFlag:
@@ -835,8 +1002,13 @@ class Scene(QtGui.QGraphicsView):
                     if not event.modifiers() & QtCore.Qt.ShiftModifier:
                         for it in self.__topology.items():
                             it.setSelected(False)
+                    if item.zValue() < 0:
+                        item.setFlag(item.ItemIsSelectable, True)
                     item.setSelected(True)
                     self.showContextualMenu()
+                    if item.zValue() < 0:
+                        item.setFlag(item.ItemIsSelectable, False)
+
                 else:
                     self.showContextualMenu()
             # When more than one item is selected display the contextual menu even if mouse is not above an item
@@ -847,7 +1019,7 @@ class Scene(QtGui.QGraphicsView):
             note.setPos(self.mapToScene(event.pos()))
             pos_x = note.pos().x()
             pos_y = note.pos().y() - (note.boundingRect().height() / 2)
-            note.setPos(pos_x, pos_y)      
+            note.setPos(pos_x, pos_y)
             command = undo.AddItem(self.__topology, note, translate("Scene", "annotation"))
             self.__topology.undoStack.push(command)
             note.editText()
@@ -877,11 +1049,11 @@ class Scene(QtGui.QGraphicsView):
         """ If The Left and Right mouse buttons are not still pressed TOGETHER and neither is the middle button
             this means the user is no longer trying to drag the scene
         """
-        
+
         if self.sceneDragging and not event.buttons() == (QtCore.Qt.LeftButton | QtCore.Qt.RightButton) and not event.buttons() & QtCore.Qt.MidButton:
             self.sceneDragging = False
             globals.GApp.scene.setCursor(QtCore.Qt.ArrowCursor)
-        else:   
+        else:
             QtGui.QGraphicsView.mouseReleaseEvent(self, event)
 
     def mouseDoubleClickEvent(self, event):
@@ -891,6 +1063,13 @@ class Scene(QtGui.QGraphicsView):
             item.setSelected(True)
             if (isinstance(item, IOSRouter) or isinstance(item, AnyEmuDevice)) and item.isStarted():
                 self.slotConsole()
+            elif isinstance(item, AnyVBoxEmuDevice) and (item.isStarted() or item.isSuspended()) and not globals.addingLinkFlag:
+                if item.get_config()['console_support']:
+                    self.slotConsole()
+                elif item.get_config()['headless_mode']:
+                    print translate("Scene", "Warning, you are using headless mode without console support")
+                else:
+                    self.slotDisplayWindowFocus()
             else:
                 self.slotConfigNode()
         else:
@@ -901,7 +1080,7 @@ class Scene(QtGui.QGraphicsView):
             if so it sets the value of the scene scroll bars based on the change between
             the previous and current mouse position
         """
-        
+
         if self.sceneDragging:
             mappedGlobalPos = self.mapFromGlobal(event.globalPos())
             hBar = self.horizontalScrollBar()
